#!perl -T

use Test::More tests => 1;

BEGIN {
	use_ok( 'CatalystX::CRUD::Controller::RHTMLO' );
}

diag( "Testing CatalystX::CRUD::Controller::RHTMLO $CatalystX::CRUD::Controller::RHTMLO::VERSION, Perl $], $^X" );
